
local Chess = require('chess')
require('lang')

function erase(aTable)
  for k, v in pairs(aTable) do
    aTable[k] = nil
  end
end

local positionData = Chess.EncodePosition()
local legalMoves = Chess.GenLegal(positionData)

function isLegalMove(aMove)
  for k, v in ipairs(legalMoves) do
    if v == aMove then
      return true
    end
  end
  return false
end

local images = {}
local pieces = {}
local draggable = 0
local move = ''
local offset = {x = 0, y = 0}
local text = literals.ltWhiteToMove
local gameOver = false
local cursorGrab, cursorGrabbing
local validTargetSquares = {}
local piecesSet = {name = 'sharm', squareWidth = 125}
local margin = 43
local assetsSize= 0.2

function initPieces()
  pieces = {
    {x = 0 * piecesSet.squareWidth, y = 0 * piecesSet.squareWidth, id = 'br'},
    {x = 1 * piecesSet.squareWidth, y = 0 * piecesSet.squareWidth, id = 'bn'},
    {x = 2 * piecesSet.squareWidth, y = 0 * piecesSet.squareWidth, id = 'bb'},
    {x = 3 * piecesSet.squareWidth, y = 0 * piecesSet.squareWidth, id = 'bq'},
    {x = 4 * piecesSet.squareWidth, y = 0 * piecesSet.squareWidth, id = 'bk'},
    {x = 5 * piecesSet.squareWidth, y = 0 * piecesSet.squareWidth, id = 'bb'},
    {x = 6 * piecesSet.squareWidth, y = 0 * piecesSet.squareWidth, id = 'bn'},
    {x = 7 * piecesSet.squareWidth, y = 0 * piecesSet.squareWidth, id = 'br'},
    {x = 0 * piecesSet.squareWidth, y = 1 * piecesSet.squareWidth, id = 'bp'},
    {x = 1 * piecesSet.squareWidth, y = 1 * piecesSet.squareWidth, id = 'bp'},
    {x = 2 * piecesSet.squareWidth, y = 1 * piecesSet.squareWidth, id = 'bp'},
    {x = 3 * piecesSet.squareWidth, y = 1 * piecesSet.squareWidth, id = 'bp'},
    {x = 4 * piecesSet.squareWidth, y = 1 * piecesSet.squareWidth, id = 'bp'},
    {x = 5 * piecesSet.squareWidth, y = 1 * piecesSet.squareWidth, id = 'bp'},
    {x = 6 * piecesSet.squareWidth, y = 1 * piecesSet.squareWidth, id = 'bp'},
    {x = 7 * piecesSet.squareWidth, y = 1 * piecesSet.squareWidth, id = 'bp'},
    {x = 0 * piecesSet.squareWidth, y = 6 * piecesSet.squareWidth, id = 'wp'},
    {x = 1 * piecesSet.squareWidth, y = 6 * piecesSet.squareWidth, id = 'wp'},
    {x = 2 * piecesSet.squareWidth, y = 6 * piecesSet.squareWidth, id = 'wp'},
    {x = 3 * piecesSet.squareWidth, y = 6 * piecesSet.squareWidth, id = 'wp'},
    {x = 4 * piecesSet.squareWidth, y = 6 * piecesSet.squareWidth, id = 'wp'},
    {x = 5 * piecesSet.squareWidth, y = 6 * piecesSet.squareWidth, id = 'wp'},
    {x = 6 * piecesSet.squareWidth, y = 6 * piecesSet.squareWidth, id = 'wp'},
    {x = 7 * piecesSet.squareWidth, y = 6 * piecesSet.squareWidth, id = 'wp'},
    {x = 0 * piecesSet.squareWidth, y = 7 * piecesSet.squareWidth, id = 'wr'},
    {x = 1 * piecesSet.squareWidth, y = 7 * piecesSet.squareWidth, id = 'wn'},
    {x = 2 * piecesSet.squareWidth, y = 7 * piecesSet.squareWidth, id = 'wb'},
    {x = 3 * piecesSet.squareWidth, y = 7 * piecesSet.squareWidth, id = 'wq'},
    {x = 4 * piecesSet.squareWidth, y = 7 * piecesSet.squareWidth, id = 'wk'},
    {x = 5 * piecesSet.squareWidth, y = 7 * piecesSet.squareWidth, id = 'wb'},
    {x = 6 * piecesSet.squareWidth, y = 7 * piecesSet.squareWidth, id = 'wn'},
    {x = 7 * piecesSet.squareWidth, y = 7 * piecesSet.squareWidth, id = 'wr'}
  }
end

function imagePath(aImageName)
  return string.format('pictures/%s/%s.png', piecesSet.name, aImageName)
end

function love.load(arg)

  for i, param in ipairs(arg) do
    local option, value = string.match(param, '--(%w+)=(%w+)')
    if string.lower(option) == 'pieces' then
      if string.lower(value) == 'smirf' then
        piecesSet = {name = 'smirf', squareWidth = 36}
      elseif string.lower(value) == 'warlord' then
        piecesSet = {name = 'warlord', squareWidth = 48}
      end
      love.window.setMode(8 * piecesSet.squareWidth + 2 * margin, 8 * piecesSet.squareWidth + 2 * margin + 20)
    elseif string.lower(option) == 'lang' then
      setLanguage(value)
      text = literals.ltWhiteToMove
    end
  end
  
  images['board'] = love.graphics.newImage(imagePath('board'))
  local colors, types = 'wb', 'pnbrqk'
  for i = 1, #colors do
    for j = 1, #types do
      local name = string.sub(colors, i, i) .. string.sub(types, j, j)
      images[name] = love.graphics.newImage(imagePath(name))
    end
  end
  images['valid'] = love.graphics.newImage(imagePath('valid'))
  love.graphics.setBackgroundColor(1, 0, 1)
  
  cursorGrab = love.mouse.newCursor('pictures/grab.png', 0, 0)
  cursorGrabbing = love.mouse.newCursor('pictures/grabbing.png', 0, 0)
  love.mouse.setCursor(cursorGrab)
  
  initPieces()
end

function screenToBoard(aX, aY)
  return math.floor(aX / piecesSet.squareWidth) + 1, 8 - math.floor(aY / piecesSet.squareWidth)
end

function boardToScreen(aX, aY)
  return piecesSet.squareWidth * (aX - 1), piecesSet.squareWidth * (8 - aY)
end

function align(aCoord)
  return math.floor(aCoord / piecesSet.squareWidth + 0.5) * piecesSet.squareWidth
end

function afterMoveProcedure()
  if positionData.halfmoveClock == 50 then
    text = literals.ltDraw
    gameOver = true
  else
    legalMoves = Chess.GenLegal(positionData)
    --print(table.concat(legalMoves, '\n') .. '\n')
    local extraPositionData = Chess.Think(positionData)
    if #legalMoves == 0 then
      text = extraPositionData.check and literals.ltCheckmate or literals.ltStalemate
      if extraPositionData.check then
        text = string.format(
          '%s%s',
          text,
          positionData.activeColor == 'w' and literals.ltBlackWins or literals.ltWhiteWins
        )
      end
      gameOver = true
    else
      text = string.format(
        '%s%s',
        extraPositionData.check and literals.ltCheck or '',
        positionData.activeColor == 'w' and literals.ltWhiteToMove or literals.ltBlackToMove
      )
    end
  end
end

function love.update(dt)
  if gameOver then
    return
  end
  
  local mx, my = love.mouse.getPosition()
  
  mx = mx - margin + 16 
  my = my - margin + 16
  
  if draggable == 0 then
    if love.mouse.isDown(1) then
      for k, v in ipairs(pieces) do
        if Chess.InRange(mx, v.x, v.x + piecesSet.squareWidth - 1) and Chess.InRange(my, v.y, v.y + piecesSet.squareWidth - 1) then
          draggable = k
          offset.x, offset.y = mx - v.x, my - v.y
          break
        end
      end
      if draggable ~= 0 then
        move = Chess.SquareToStr(screenToBoard(mx, my))
        love.mouse.setCursor(cursorGrabbing)
        erase(validTargetSquares)
        for k, v in ipairs(legalMoves) do
          if string.sub(v, 1, 2) == move then
            local x, y = boardToScreen(Chess.StrToSquare(string.sub(v, 3, 4)))
            validTargetSquares[#validTargetSquares + 1] = {x = x, y = y}
          end
        end
      end
    end
  end
  
  if draggable ~= 0 then
    if not love.mouse.isDown(1) then
      pieces[draggable].x = align(pieces[draggable].x)
      pieces[draggable].y = align(pieces[draggable].y)
      
      move = move .. Chess.SquareToStr(screenToBoard(pieces[draggable].x, pieces[draggable].y))
      
      if isLegalMove(move) then
        local result, square = Chess.IsEnPassant(positionData, move)
        if result then
          for k, v in ipairs(pieces) do
            if Chess.SquareToStr(screenToBoard(v.x, v.y)) == square then
              v.x = 8 * piecesSet.squareWidth--320
            end
          end
        end
        
        local result, y1, x1, x2, x3, x4 = Chess.IsCastling(positionData, move)
        if result then
          for k, v in ipairs(pieces) do
            if Chess.InRange(v.x, 0, 7 * piecesSet.squareWidth) then--280) then
              local x, y = screenToBoard(v.x, v.y)
              if (x == x2) and (y == y1) then
                if string.sub(v.id, 2, 2) == 'k' then
                  v.x, v.y = boardToScreen(x3, y1)
                end
                if string.sub(v.id, 2, 2) == 'r' then
                  v.x, v.y = boardToScreen(x4, y1)
                end
              end
            end
          end
        end
        
        if Chess.IsPromotion(positionData, move) then
          local myPromotion = 'q'
          move = move .. myPromotion
          pieces[draggable].id = string.sub(pieces[draggable].id, 1, 1) .. myPromotion
        end
        
        Chess.DoMove(positionData, Chess.StrToMove(move))
        print(Chess.DecodePosition(positionData) .. '\n')
        print(Chess.BoardToText(positionData.piecePlacement) .. '\n')
        afterMoveProcedure()
        
        for k, v in ipairs(pieces) do
          if k ~= draggable then
            if (v.x == pieces[draggable].x) and (v.y == pieces[draggable].y) then
              v.x = 8 * piecesSet.squareWidth--320
            end
          end
        end
      else
        local x, y = Chess.StrToSquare(string.sub(move, 1, 2))
        pieces[draggable].x, pieces[draggable].y = boardToScreen(x, y)
        print(string.format('illegal move (%s)', move) .. '\n')
      end
      
      draggable = 0
      love.mouse.setCursor(cursorGrab)
    end
  end
  
  if draggable ~= 0 then
    pieces[draggable].x, pieces[draggable].y = mx - offset.x, my - offset.y
    if pieces[draggable].x < 0 - 2 then pieces[draggable].x = 0 - 2 end
    if pieces[draggable].y < 0 - 2 then pieces[draggable].y = 0 - 2 end
    --~ if pieces[draggable].x > 280 + 2 then pieces[draggable].x = 280 + 2 end
    --~ if pieces[draggable].y > 280 + 2 then pieces[draggable].y = 280 + 2 end
    if pieces[draggable].x > 7 * piecesSet.squareWidth + 2 then pieces[draggable].x = 7 * piecesSet.squareWidth + 2 end
    if pieces[draggable].y > 7 * piecesSet.squareWidth + 2 then pieces[draggable].y = 7 * piecesSet.squareWidth + 2 end
  end
end

function love.keypressed(key)
  if key == "escape" then
    love.event.quit()
  end
end

function love.draw()
  love.graphics.setColor(5/8, 5/8, 5/8, 1)
  love.graphics.rectangle('fill', 0, 0, 8 * piecesSet.squareWidth + 2 * margin, 8 * piecesSet.squareWidth + 2 * margin,0, assetsSize)
  --~ love.graphics.draw(images['board'], 0, 0)
  love.graphics.setColor(1, 1, 1, 1)
  love.graphics.draw(images['board'], margin, margin,0, assetsSize*0.985)
  if draggable ~= 0 then
    for k, v in ipairs(validTargetSquares) do
      love.graphics.draw(images['valid'], v.x + margin, v.y + margin,0, assetsSize)
    end
  end
  for k, v in ipairs(pieces) do
    if k ~= draggable then
      --~ if Chess.InRange(v.x, 0, 280) then
      if Chess.InRange(v.x, 0, 7 * piecesSet.squareWidth) then
        love.graphics.draw(images[v.id], v.x + margin, v.y + margin - 85,0, assetsSize)
      end
    end
  end
  if draggable ~= 0 then
    love.graphics.draw(images[pieces[draggable].id], pieces[draggable].x + margin, pieces[draggable].y + margin -85,0, assetsSize)
  end
  --~ love.graphics.print({{0, 0, 0}, text}, 0, 320, 0, 1, 1, -3, -3)
  --~ love.graphics.print({{0, 0, 0}, text}, 0, 8 * piecesSet.squareWidth, 0, 1, 1, -3, -3)
  love.graphics.setColor(7/8, 7/8, 7/8, 1)
  love.graphics.rectangle('fill', 0, 8 * piecesSet.squareWidth + 2 * margin, 8 * piecesSet.squareWidth + 2 * margin, 20)
  love.graphics.print({{0, 0, 0}, text}, 0, 8 * piecesSet.squareWidth + 2 * margin, 0, 1, 1, -3, -3)
end


function love.touchpressed(id, x, y, dx, dy, pressure)
  if gameOver then
    return
  end
  
  x = x - margin + 16 
  y = y - margin + 16
  
  if draggable == 0 then
    for k, v in ipairs(pieces) do
      if Chess.InRange(x, v.x, v.x + piecesSet.squareWidth - 1) and Chess.InRange(y, v.y, v.y + piecesSet.squareWidth - 1) then
        draggable = k
        offset.x, offset.y = x - v.x, y - v.y
        break
      end
    end
    if draggable ~= 0 then
      move = Chess.SquareToStr(screenToBoard(x, y))
      erase(validTargetSquares)
      for k, v in ipairs(legalMoves) do
        if string.sub(v, 1, 2) == move then
          local x, y = boardToScreen(Chess.StrToSquare(string.sub(v, 3, 4)))
          validTargetSquares[#validTargetSquares + 1] = {x = x, y = y}
        end
      end
    end
  end
end

function love.touchmoved(id, x, y, dx, dy, pressure)
  if draggable ~= 0 then
    x = x - offset.x
    y = y - offset.y
    if x < 0 - 2 then x = 0 - 2 end
    if y < 0 - 2 then y = 0 - 2 end
    if x > 7 * piecesSet.squareWidth + 2 then x = 7 * piecesSet.squareWidth + 2 end
    if y > 7 * piecesSet.squareWidth + 2 then y = 7 * piecesSet.squareWidth + 2 end
    pieces[draggable].x, pieces[draggable].y = x, y
  end
end

function love.touchreleased(id, x, y, dx, dy, pressure)
  if draggable ~= 0 then
    pieces[draggable].x = align(pieces[draggable].x)
    pieces[draggable].y = align(pieces[draggable].y)
    
    move = move .. Chess.SquareToStr(screenToBoard(pieces[draggable].x, pieces[draggable].y))
    
    if isLegalMove(move) then
      -- Perform the move
      local result, square = Chess.IsEnPassant(positionData, move)
      if result then
        for k, v in ipairs(pieces) do
          if Chess.SquareToStr(screenToBoard(v.x, v.y)) == square then
            v.x = 8 * piecesSet.squareWidth--320
          end
        end
      end
      
      local result, y1, x1, x2, x3, x4 = Chess.IsCastling(positionData, move)
      if result then
        for k, v in ipairs(pieces) do
          if Chess.InRange(v.x, 0, 7 * piecesSet.squareWidth) then
            local x, y = screenToBoard(v.x, v.y)
            if (x == x2) and (y == y1) then
              if string.sub(v.id, 2, 2) == 'k' then
                v.x, v.y = boardToScreen(x3, y1)
              end
              if string.sub(v.id, 2, 2) == 'r' then
                v.x, v.y = boardToScreen(x4, y1)
              end
            end
          end
        end
      end
      
      if Chess.IsPromotion(positionData, move) then
        local myPromotion = 'q'
        move = move .. myPromotion
        pieces[draggable].id = string.sub(pieces[draggable].id, 1, 1) .. myPromotion
      end
      
      Chess.DoMove(positionData, Chess.StrToMove(move))
      print(Chess.DecodePosition(positionData) .. '\n')
      print(Chess.BoardToText(positionData.piecePlacement) .. '\n')
      afterMoveProcedure()
      
      for k, v in ipairs(pieces) do
        if k ~= draggable then
          if (v.x == pieces[draggable].x) and (v.y == pieces[draggable].y) then
            v.x = 8 * piecesSet.squareWidth--320
          end
        end
      end
      
      -- Update the game state to switch turns
      if positionData.activeColor == 'w' then
        text = literals.ltBlackToMove
      else
        text = literals.ltWhiteToMove
      end
      
    else
      -- Move the piece back to its original position
      local bx, by = Chess.StrToSquare(string.sub(move, 1, 2))
      pieces[draggable].x, pieces[draggable].y = boardToScreen(bx, by)
      print(string.format('illegal move (%s)', move) .. '\n')
    end
    
    draggable = 0
  end
end

